// This source code is subject to the terms of the Mozilla Public License 2.0 at https://mozilla.org/MPL/2.0/
// © kocurekc

//@version=4
//
// @author Kocurekc
// Rev-2, Updates for Bernoulli calc, added functions, averaging methods, Purest Calcs 
// Rev-1, Added Alert for UP/DOWN triangle 
// Rev-0, Bernoulli Process,  Information Measurement
// Binary Entropy Function
//
// Thanks to @Picte for his inspiring idea STOCH-HVP(Balipour)
// Live a little, publish your scripts...be a John Elhers
//
study("Bernoulli - V2", overlay=false, shorttitle="Bernoulli", precision=2)

//so many ways to measure the same thing...except acceleration, that one is neat
measure(_type, _src, _lbl) =>
    float result = 0
    if _type=="Value"                                           //Average value over lookback
        result := _src
    if _type=="SMA"                                             //Average value over lookback
        result := sma(_src,_lbl)
    if _type=="Change"                                          //Average difference (1-bar) over lookback
        result := sma(change(_src),_lbl)
    if _type=="Momentum"                                        //Average difference (lookback bars) over lookback
        result := sma(change(_src,_lbl),_lbl)
    if _type=="Acceleration"                                    //Average Change of the Change over lookback, average acceleration 
        result := sma(change(change(_src,_lbl),_lbl),_lbl)
    if _type=="Contribution"                                    //Today ratiod to total over lookback, how much did this bar contirbute to the total
        result := _src/sum(_src,_lbl)
    if _type=="%-Change"                                        //Percent Change over lookback
        result := sma(change(_src)/nz(_src[_lbl],_src),_lbl)
    if _type=="Volatility"                                      //Natural Log Volatility adjusted
        Vaa = log(_src/nz(_src[1],_src))
        Vam = sma(Vaa,_lbl)
        result := sqrt(sum(pow(Vaa-Vam,2),_lbl)/_lbl)
    result

    
//Returns the normal distribution CDF
n_CDF(_src,_len,_smo) =>
    x_bar = sma(_src,_len) //mean
    s = sqrt(sum(pow(_src - x_bar,2),_len)/(_len-1))
    z_bar = max(min((_src - x_bar)/s,5.55),-5.55)
    CDF = sma((1/(1+pow((1-z_bar/5.555),(1/0.1186)))),_smo)
    
//Returns the Burn, Bernoulli calc
bern(_src,_avg,_len,_xn) =>
    _r2 = min(max(percentrank(_src,_avg)/100,0.001),0.999)
    burn = sum(((_xn?-1:1)*_r2*log(_r2)/log(2)) - (1-_r2)*log(1-_r2)/log(2),_len)
    

src = close
len = input(22, title="Entropy Length", type=input.integer)
range = input(0.67, title="Trade Band - confirmation", type=input.float, step=0.1)
avg = input(88, title="Averaging Length", type=input.integer)
type = input("Contribution", "Measurement Type", options = ["Value", "SMA", "Change", "%-Change", "Momentum", "Acceleration", "Contribution", "Volatility"])

vPR = input(5, title="Percent Rank Limit", type=input.integer)
bc = input(true, title="Include Source", type=input.bool)
vc = input(true, title="Include Volume", type=input.bool)
pb = input(true, title="Print Bands", type=input.bool)
xc = input(false, title="Extra Highlighting", type=input.bool)
xn = input(false, title="Bernoulli Purest", type=input.bool)

smo = input(3, title="Probability Smoother", type=input.integer, minval=1)


//Entropy Calculation, Bernoulli Process source (close) or for Volume or both
cr = measure(type,src,len)
vr = measure(type,log(volume),len)
cr2 = xn?n_CDF(cr,len,smo):cr
vr2 = xn?n_CDF(vr,len,smo):vr
infoc = bc?bern(cr2,avg,len,xn):0
infov = vc?bern(vr2,avg,len,xn):0
info2 = (bc and vc) ? (infoc - infov): xn ? (0 + (infoc + infov)/(-len + 1)):(infoc + infov)

hvp = percentrank(info2,avg)

//Plotting 
col1 = info2>(range) ? color.green : info2<(-range) ? color.red:color.gray
col2 = info2>(range) and hvp>(100-vPR)? color.maroon : info2<(-range) and hvp<vPR ? color.lime: info2>(range) ? color.green : info2<(-range) ? color.red: color.gray
plot(info2,title="Bernoulli",style=plot.style_columns, color=xc?col2:col1)
plot((pb and bc and vc and xn) ? na : (pb and bc and vc) ? infoc  : na, title="price", color=color.blue)
plot((pb and bc and vc and xn) ? na : (pb and bc and vc) ? -infov : na, title="volume", color=color.orange)

//Percent Rank and ploting
plotshape((hvp>(100-vPR) and (abs(info2) > abs(range))) ? info2 : na, location=location.absolute, style=shape.triangledown, color=color.new(color.red, transp=50), size=size.tiny)
plotshape(hvp<vPR and (abs(info2) > abs(range)) ? info2 : na, location=location.absolute, style=shape.triangleup, color=color.new(color.green, transp=50), size=size.tiny)

//Alerts
downAlert = (hvp[1] < (100-vPR)) and (hvp>(100-vPR)) ? true:false
upAlert = (hvp[1] > vPR) and (hvp<vPR) ? true:false
alertcondition(downAlert, title='Alert on Down Triangle', message='Bernoulli Down Triangle!')
alertcondition(upAlert, title='Alert on Up Triangle', message='Bernoulli Up Triangle!')